/**
 * Cache for radio data to make updates easier to work with.
 */
 function RadioDataCache(){
   //Information about the channels by stationUID
  this.data=[];
  //Information about the station information
  this.stationData=[]
    return this;
}
RadioDataCache.updatable=[
  "smallImageURL","largeImageURL","songPlayTime",
  "song","artist","album",
  "amazonUrl","itunesurl","album","streamurl"
  ];
/**
* This function updates the modifiable fields
* of channelData segment c with the updated values in 
* x
*/
RadioDataCache.prototype.realApplyUpdate=function(c,updated){

 RadioDataCache.updatable.each(function(i){
//    DEBUG(i + "|" + c[i]  + "->" + updated[i]);
     c[i]=updated[i];
  });
}

RadioDataCache.prototype.copyStationInfo=function(x){
  var res={
    id :x.id,
    call_letters:x.call_letters,
    band : x.band,
    state : x.state,
    zipcode: x.zipcode,
    station_art_url: x.station_art_url,
    frequency: x.frequency,
    is_digital:x.is_digital
  };
  return res;
}

RadioDataCache.prototype.applyUpdate=function(data,x,func){
  //  DEBUG("FUNC " + x.id + " | Channels=" + x.channels.length )
  var self=this;   
  x.channels.each(function(y){
    //      var ent=data.channelMap[y.channelNumber];
    var ent=self.data[y.stationUID];
    if(ent){
      self.realApplyUpdate(ent,y);
      y=ent;
    }else{
      DEBUG("Pushing New Data for " +  y.stationUID +  "!");
      self.data[y.stationUID]=y;//add new channel
      y.stationId=x.id;
      self.stationData[x.id]=self.copyStationInfo(x);//add the station information

    }
    if (func) func(self.stationData[y.stationId],y);
  });
};

//Map channel numbers to channel ids
RadioDataCache.prototype.buildChannelMap=function(x,sdata){
  var ret=[];
    for(var i=0;i<x.channels.length;++i){
      //Map channel numbers to channels to channels.
     var c=x.channels[i];
     this.data[c.stationUID]=c;
     c.stationId=sdata.id;
     ret.push(c);
    }
   return ret;
};

/**
* Get Element out of cache 
* @returns [stationdata,channelData]
*/
RadioDataCache.prototype.getByUID=function(uid){
    var elt=this.data[uid];
	  //DEBUG("Elt is +" + dumpObj(elt)+"|");
    if(elt ){
	return [this.stationData[elt.stationId],elt];
    }else{
	return null;
    }
};

/**
* Apply an update and invoke the function with 
* the station and channel information for each element updated
*/
RadioDataCache.prototype.update=function(newdata,func,fulldata){
  var obj=this;
//  DEBUG("Applying Update");
  newdata.each(function(x){

    if(x.channels){
      var existing=obj.stationData[x.id];
      if(existing && !fulldata){
        obj.applyUpdate(existing,x,func);
      }else{
        var sdata=obj.stationData[x.id]=obj.copyStationInfo(x);
        var ret=obj.buildChannelMap(x,sdata);
        if(func){
          ret.each(function(y){
                        DEBUG("FUNC " + y.id + " | Channels=" + y )
            func(sdata,y);
          });
        }
      }
      x = null;
    }else{
//      DEBUG("Delta Update");
      //No channels so this is a delta version.
      var existing=obj.data[x.stationUID];
      if(existing){
        //If we have the reset of the information we can make a reasonable request
 //       DEBUG("Applying Real Update" + existing.stationUID + " -> " + dumpObj(x));
        obj.realApplyUpdate(existing,x);
        func(obj.stationData[existing.stationId],existing);
      
      }

    }
  });
  obj = null;
  newdata=null;
};

RadioDataCache.prototype.clear=function(){
  this.data=[];
  this.stationdata=[];
};
